from strawberry.types import ExecutionContext
from strawberry.types.graphql import OperationType


def is_mutation(execution_context: ExecutionContext) -> bool:
    try:
        return execution_context.operation_type == OperationType.MUTATION
    except RuntimeError as _:
        return False



import typing as T
import functools
import inspect
from enum import Enum
from pydantic import BaseModel
from edge_orm import Resolver, Node
from edge_orm.types_generator.main import COUNT_POSTFIX
from ..strawberry_utils import StrawMixin
from strawberry.types.nodes import SelectedField
from app import helpers


NodeType = T.TypeVar("NodeType", bound=Node)
REZ_FUNC = T.Callable[..., T.Union[Resolver, T.Awaitable[Resolver]]]
UPDATED_REZ_FUNC = T.Callable[
    ..., T.Union[Resolver, T.Awaitable[Resolver], None, T.Awaitable[None]]
]
Func = T.TypeVar("Func", bound=T.Callable[..., T.Any])


def kwargs_from_selected_field(
    rez_function: T.Callable[..., T.Any],
    selected_field: SelectedField,
    auth_id: str | None,
) -> dict[str, T.Any]:
    params = inspect.signature(rez_function).parameters
    kwargs = {helpers.camel_to_snake(k): v for k, v in selected_field.arguments.items()}
    if "_auth_id" in params and "_auth_id" not in kwargs:
        kwargs["_auth_id"] = auth_id
    if "_child_field" in params and "_child_field" not in kwargs:
        kwargs["_child_field"] = selected_field
    return kwargs


class PreloadRaw(BaseModel):
    field_name: str
    update_resolver_f: T.Callable[..., T.Any] | None = None
    return_type: T.Any | None = None
    path_to_return_type: list[str] | None = None
    fields_to_include: set[str] | None = None
    alias: str | None = None
    rez_function: T.Callable[..., T.Any] | None = None
    permissions: set[Enum] | None = None

    def only_fields_to_include(self) -> bool:
        return {"field_name", "fields_to_include"} == self.dict(
            exclude_none=True
        ).keys()

    async def update_rez(
        self,
        rez: Resolver,
        selected_field: SelectedField,
        return_type: T.Type[StrawMixin[NodeType]] | T.Any,
        auth_id: str = None,
    ) -> Resolver | None:
        """Assuming update_resolver_f does not exist"""
        """returns none if first_name and only including fields"""
        """returns resolver if return type is a straw mixin"""
        has_count_postfix = self.field_name.endswith(COUNT_POSTFIX)
        if self.update_resolver_f:
            raise Exception("update_resolver_f already exists, just use that.")
        if self.fields_to_include:
            rez.include_fields(*self.fields_to_include)
        if (
            not (inspect.isclass(return_type) and issubclass(return_type, StrawMixin))
            and not self.rez_function
            and not has_count_postfix
        ):
            return None
        edge_name = self.alias or self.field_name
        if self.rez_function:
            kwargs = kwargs_from_selected_field(
                rez_function=self.rez_function,
                selected_field=selected_field,
                auth_id=auth_id,
            )
            nested_rez = self.rez_function(**kwargs)
            if inspect.isawaitable(nested_rez):
                nested_rez = await nested_rez
        else:
            nested_rez = return_type.Config.resolver_type()
            if has_count_postfix:
                nested_rez.is_count = True
        result = getattr(rez, edge_name)(nested_rez)
        if inspect.isawaitable(result):
            await result
        return nested_rez


class SkipTo(BaseModel):
    return_type: T.Any
    path_to_return_type: list[str]


class Preload(BaseModel):
    """
    update_resolver_f can be null in cases where you just want to go to return type

    skip_to only given if return type is Straw or if you should 'continue'.
    Otherwise, you can stop following this trail.
    """

    update_resolver_f: T.Callable[..., T.Any] | None
    update_resolver_f_given: bool

    skip_to: SkipTo
    fields_to_include: set[str]
    permissions: set[Enum] | None


def preload(
    *,
    update_resolver_f: UPDATED_REZ_FUNC = None,
    return_type: T.Any = None,
    path_to_return_type: list[str] = None,
    fields_to_include: set[str] = None,
    alias: str = None,
    rez_function: REZ_FUNC = None,
    permissions: T.Optional[set[Enum]] = None
) -> T.Callable[[Func], Func]:
    def inner(func: Func) -> Func:
        preload_raw = PreloadRaw(
            field_name=func.__name__,
            update_resolver_f=update_resolver_f,
            return_type=return_type,
            path_to_return_type=path_to_return_type,
            fields_to_include=fields_to_include,
            alias=alias,
            rez_function=rez_function,
            permissions=permissions,
        )
        if "raw_preloads" not in func.__dict__:
            func.__dict__["raw_preloads"] = []
        func.__dict__["raw_preloads"].append(preload_raw)
        return func

    return inner


def is_count(func):  # type: ignore
    @functools.wraps(func)
    async def wrapper(*args, **kwargs):  # type: ignore
        rez = func(*args, **kwargs)
        if inspect.isawaitable(rez):
            rez = await rez
        rez.is_count = True
        return rez

    return wrapper



from pathlib import Path
from enum import Enum
import logging
from pydantic import BaseSettings
from . import logs

parent_dir = Path(__file__).absolute().parent
base_dir = parent_dir.parent

logger = logs.create_logger(name="config", level=logging.DEBUG)


class Env(str, Enum):
    SANDBOX = "sandbox"
    LIVE = "live"


class Platform(str, Enum):
    LOCAL = "local"

    CLOUD_RUN = "cloudrun"
    RAILWAY = "railway"

    APP_RUNNER = "app_runner"
    LAMBDA = "lambda"
    HEROKU = "heroku"

    PORTER_AWS = "porter_aws"
    PORTER_GCP = "porter_gcp"
    PORTER_DO = "porter_do"

    ZEET_CLOUDRUN = "zeet_cloudrun"


class AppSettings(BaseSettings):
    api_prefix: str = "/api/v1"
    app_name_prefix: str = "quilt"
    version: str = "0.1.0"


class SentrySettings(BaseSettings):
    sentry_dsn: str


class Settings(AppSettings, SentrySettings):
    env: Env
    branch: str

    platform: Platform
    base_dir: Path = base_dir

    @property
    def app_name(self) -> str:
        return f"{self.app_name_prefix} {self.branch}-{self.env.value} on {self.platform.value}"

    def is_live(self) -> bool:
        return self.env == Env.LIVE

    def is_sandbox(self) -> bool:
        return self.env == Env.SANDBOX



import os
import typing as T
import httpx
from pydantic import BaseModel
from quilt.errors import DisplayException

BASE_URL = "https://maps.googleapis.com/maps/api/place"

google_places_api_key = os.environ["GOOGLE_PLACES_API_KEY"]


class GooglePlacesException(Exception):
    pass


class GooglePlacesExceptionDisplay(DisplayException):
    pass


class Component(BaseModel):
    long_name: str
    short_name: str
    types: list[str]


class Point(BaseModel):
    lat: float
    lng: float


class Viewport(BaseModel):
    northeast: Point
    southwest: Point


class Geometry(BaseModel):
    location: Point
    viewport: Viewport


class Photo(BaseModel):
    photo_reference: str
    height: int
    width: int
    html_attributions: list[str]


class Address(BaseModel):
    street_address: str | None = None
    city: str | None = None
    state_abbr: str | None = None
    postal_code: str | None = None
    country_name: str | None = None

    @classmethod
    def from_place(cls, place: "Place") -> T.Optional["Address"]:
        if not place.adr_address:
            return None
        chunks = place.adr_address.split('<span class="')
        d = {}
        for chunk in chunks:
            if not chunk:
                continue
            try:
                label = chunk[: chunk.index('"')]
                value = chunk[chunk.index(">") + 1 : chunk.index("<")]
                d[label] = value
            except ValueError as e:
                print(f"Error With Location {e=}, {chunk=}")
        return cls(
            street_address=d.get("street-address"),
            city=d.get("locality"),
            state_abbr=d.get("region"),
            postal_code=d.get("postal-code"),
            country_name=d.get("country-name"),
        )


class Place(BaseModel):
    address_components: list[Component]
    adr_address: str
    formatted_address: str
    geometry: Geometry
    icon: str
    icon_background_color: str
    icon_mask_base_uri: str
    name: str
    photos: list[Photo] = []
    place_id: str
    reference: str
    url: str
    utc_offset: int
    vicinity: str

    _endpoint: T.ClassVar[str] = BASE_URL + "/details/json?"
    _photos_endpoint: T.ClassVar[str] = BASE_URL + "/photo"

    @classmethod
    async def get(cls, place_id: str) -> "Place":
        async with httpx.AsyncClient() as client:
            response = await client.get(
                cls._endpoint,
                params={"place_id": place_id, "key": google_places_api_key},
            )
        data = response.json()
        if data["status"] == "INVALID_REQUEST":
            raise GooglePlacesExceptionDisplay(
                f"The place you are looking for was not found: place_id {place_id} not found."
            )
        return Place(**data["result"])

    async def get_photo_urls(self) -> list[str]:
        """returns google urls of the photos"""
        urls: list[str] = []
        async with httpx.AsyncClient() as client:
            for photo in self.photos:
                response = await client.get(
                    self._photos_endpoint,
                    params={
                        "photo_reference": photo.photo_reference,
                        "key": google_places_api_key,
                        "maxwidth": photo.width,
                        "maxheight": photo.height,
                    },
                )
                if response.status_code != 302:
                    raise GooglePlacesException(
                        f"Invalid photo reference: {photo.photo_reference=}"
                    )
                urls.append(response.headers["location"])
        return urls



import typing as T
from datetime import datetime
from zoneinfo import ZoneInfo
import strawberry
from strawberry.unset import UnsetType


@strawberry.interface
class InputDateTimeInterface:
    day: int
    month: int
    year: int
    hour: int
    minute: int

    def to_datetime(self, tz: ZoneInfo) -> datetime:
        return datetime(
            day=self.day,
            month=self.month,
            year=self.year,
            hour=self.hour,
            minute=self.minute,
            tzinfo=tz,
        )

    @classmethod
    def from_datetime(
        cls, dt: datetime, zone_info: ZoneInfo
    ) -> "InputDateTimeInterface":
        dt = dt.astimezone(tz=zone_info)
        return cls(
            day=dt.day, month=dt.month, year=dt.year, hour=dt.hour, minute=dt.minute
        )


@strawberry.input
class InputDateTime(InputDateTimeInterface):
    pass

    @staticmethod
    def to_datetime_or_none(
        input: T.Union["InputDateTime", None, UnsetType], tz: ZoneInfo
    ) -> datetime | None | UnsetType:
        if isinstance(input, InputDateTime):
            return input.to_datetime(tz)
        return input


@strawberry.type
class InputDateTimeDisplay(InputDateTimeInterface):
    pass



import typing as T
from fastapi import Request
from starlette.requests import URL  # type: ignore


class RequestMixin:
    request: Request

    def get_header_from_list(
        self, name: str, first_val: T.Optional[bool] = True
    ) -> str | None:
        arr = self.request.headers.get(name)
        if arr:
            if not first_val:
                return arr
            return arr.split(",")[-1]
        return None

    @property
    def user_agent(self) -> str | None:
        return self.request.headers.get("user-agent")

    @property
    def x_forwarded_for(self) -> str | None:
        return self.get_header_from_list("x-forwarded-for")

    @property
    def cf_connecting_ip(self) -> str | None:
        return self.get_header_from_list("cf-connecting-ip")

    @property
    def ip_address(self) -> str | None:
        if cf_ip := self.cf_connecting_ip:
            return cf_ip
        if ip := self.x_forwarded_for:
            return ip
        return self.request.client.host

    @property
    def url(self) -> URL:
        return self.request.url

    @property
    def url_used(self) -> str:
        return str(self.url)


class RequestManagerException(Exception):
    pass


class RequestManager(RequestMixin):
    def __init__(self, request: Request):
        if request is None:
            raise RequestManagerException("Request cannot be None.")
        self.request = request



import typing as T
import types
from edge_orm import Node
from app import helpers
from ..strawberry_utils import StrawMixin
from strawberry.types.nodes import SelectedField, FragmentSpread, InlineFragment


StrawType = T.TypeVar("StrawType", bound=StrawMixin[Node])


class ResolverUtilException(Exception):
    pass


def base_type_from_return_type(return_type: T.Any) -> T._AnnotatedAlias | None:  # type: ignore
    if isinstance(return_type, T._AnnotatedAlias):  # type: ignore
        return return_type
    if isinstance(return_type, (T._UnionGenericAlias, types.UnionType)):  # type: ignore
        args = T.get_args(return_type)
        for a in args:
            if base_type := base_type_from_return_type(a):
                return base_type
    if isinstance(return_type, (T.GenericAlias, T._GenericAlias)):  # type: ignore
        return T.get_args(return_type)[0]
    return None


def return_type_or_str_from_return_annotation(return_annotation: T.Any) -> T.Any:
    """Unpacks annotation if forward-looking"""
    while isinstance(return_annotation, T.GenericAlias):  # type: ignore
        return_annotation = T.get_args(return_annotation)[0]
    if "Annotated[ForwardRef" in str(return_annotation):
        annotated_return_type = base_type_from_return_type(return_annotation)
        if not annotated_return_type:
            raise Exception(
                f"Invalid return_type: {return_annotation=}, {type(return_annotation)}"
            )
        args = T.get_args(annotated_return_type)
        abc = args[1].resolve_forward_ref(args[0])
        return abc.resolve_type()
    return return_annotation


def return_type_from_return_annotation(
    raw_return_type: T.Any, current_straw_type: T.Type[StrawType]
) -> T.Any:
    return_type = return_type_or_str_from_return_annotation(
        return_annotation=raw_return_type
    )
    if isinstance(return_type, str) and (
        return_type == current_straw_type.__name__
        or f"['{current_straw_type.__name__}']" in str(return_type)
    ):
        return current_straw_type
    if isinstance(return_type, str):
        raise Exception(
            f"Return type cannot be str but is: {return_type=}, {raw_return_type=}, {current_straw_type=}."
        )
    return return_type


def selected_field_from_path(
    top_field: SelectedField, path: list[str]
) -> SelectedField:
    for part in path:
        part = helpers.snake_to_camel(part)
        found_part = False
        selections = top_field.selections.copy()
        for sel in selections:
            if isinstance(sel, (FragmentSpread, InlineFragment)):
                selections.extend(sel.selections)
                continue
            if sel.name == part:
                top_field = sel
                found_part = True
                break
        if found_part is False:
            message = f"{part=} not in {top_field.selections}"
            raise ResolverUtilException(message)
    return top_field



from datetime import datetime
from pydantic import AnyHttpUrl, BaseModel


class CloudinaryImage(BaseModel):
    asset_id: str
    bytes: int | None = None
    created_at: datetime
    etag: str | None = None
    format: str
    height: int
    width: int
    original_filename: str | None = None
    pages: int | None = None
    phash: str | None = None
    placeholder: str | None = None
    public_id: str
    resource_type: str
    secure_url: AnyHttpUrl | None = None
    signature: str | None = None
    tags: list[str] | None = None
    type: str | None = None
    url: AnyHttpUrl
    version: int
    version_id: str | None = None



import typing as T
import inspect
import time
from quilt import logs
from . import logic, helpers, models
from ..strawberry_utils import StrawMixin

logger = logs.create_logger(__name__)

PreloadRaw = models.PreloadRaw


RAW_PRELOADS_KEY = "raw_preloads"
PRELOADS_KEY = "preloads"


def merge_raw_preloads(
    field_name: str, raw_preload_lst: list[PreloadRaw]
) -> list[PreloadRaw]:
    if len(raw_preload_lst) <= 1:
        return raw_preload_lst
    merged_raw_pres = []
    fields_to_include_set: set[str] = set()
    for raw_pre in raw_preload_lst:
        if raw_pre.only_fields_to_include() and raw_pre.fields_to_include:
            fields_to_include_set.update(raw_pre.fields_to_include)
        else:
            merged_raw_pres.append(raw_pre)
    if merged_raw_pres:
        for i in merged_raw_pres:
            if not i.fields_to_include:
                i.fields_to_include = fields_to_include_set
            else:
                i.fields_to_include.update(fields_to_include_set)
    else:
        merged_raw_pres = [
            PreloadRaw(field_name=field_name, fields_to_include=fields_to_include_set)
        ]
    return merged_raw_pres


def get_all_subclasses(cls: T.Type[StrawMixin]):
    subclasses = []
    for subclass in cls.__subclasses__():
        subclasses.append(subclass)
        subclasses.extend(get_all_subclasses(subclass))
    return subclasses


def create_preloads() -> None:
    start = time.time()
    for cls in set(get_all_subclasses(StrawMixin)):
        for field in cls._type_definition.fields:
            straw_field = getattr(cls, field.name, None)
            if straw_field:
                d = straw_field.__dict__
                actual_return_type = helpers.return_type_from_return_annotation(
                    raw_return_type=straw_field.__annotations__["return"],
                    current_straw_type=cls,
                )
                if (
                    inspect.isclass(actual_return_type)
                    and issubclass(actual_return_type, StrawMixin)
                    and RAW_PRELOADS_KEY not in d
                ):
                    # if not in resolver, do not do this
                    if field.name in cls.Config.resolver_type._edge_resolver_map:
                        d[RAW_PRELOADS_KEY] = [
                            PreloadRaw(field_name=field.name, alias=field.name)
                        ]
                if raw_preloads := d.get(RAW_PRELOADS_KEY):
                    raw_preloads = T.cast(list[PreloadRaw], raw_preloads)
                    if PRELOADS_KEY not in d:
                        d[PRELOADS_KEY] = []

                    merged_raw_preloads = raw_preloads
                    if len(raw_preloads) > 1:
                        merged_raw_preloads = merge_raw_preloads(
                            field_name=field.name, raw_preload_lst=raw_preloads
                        )

                    for raw_preload in merged_raw_preloads:
                        # if not return_type given, get current return type
                        preload = logic.preload_raw_to_preload(
                            current_straw_type=cls,
                            preload_raw=raw_preload,
                            actual_return_type=actual_return_type,
                        )
                        d[PRELOADS_KEY].append(preload)
    took = time.time() - start
    logger.debug(f"preloaded done, took {round(took * 1000)} ms")



import typing as T
from ..strawberry_utils import StrawMixin
from .models import Preload, PreloadRaw, SkipTo
from . import helpers


class LogicException(Exception):
    pass


def preload_raw_to_preload(
    current_straw_type: T.Type[StrawMixin[T.Any]],
    preload_raw: PreloadRaw,
    actual_return_type: T.Any,
) -> Preload:
    # deal with return types first
    if preload_raw.return_type and preload_raw.path_to_return_type is None:
        raise LogicException(
            f"If return_type is provided, path_to_return_type must be provided. For {preload_raw.dict()=}."
        )
    if preload_raw.return_type:
        return_type = helpers.return_type_from_return_annotation(
            raw_return_type=preload_raw.return_type,
            current_straw_type=current_straw_type,
        )
    else:
        return_type = actual_return_type
    skip_to = SkipTo(
        return_type=return_type,
        path_to_return_type=preload_raw.path_to_return_type or [],
    )
    if preload_raw.update_resolver_f:
        update_resolver_f = preload_raw.update_resolver_f
        update_resolver_f_given = True
    else:
        update_resolver_f = preload_raw.update_rez
        update_resolver_f_given = False

    return Preload(
        update_resolver_f=update_resolver_f,
        update_resolver_f_given=update_resolver_f_given,
        skip_to=skip_to,
        fields_to_include=preload_raw.fields_to_include or set(),
        permissions=preload_raw.permissions,
    )



import os
import hmac
from fastapi import APIRouter, Header, Depends, HTTPException
from quilt.models import PhoneNumber
from app import settings, Env
from . import logic

router = APIRouter()


def is_valid_header(secret_key: str = Header(...)) -> None:
    if not hmac.compare_digest(secret_key, os.environ["CUSTOM_BEARER_SECRET_KEY"]):
        raise HTTPException(status_code=401, detail="Invalid key.")


def is_sandbox() -> None:
    if settings.env != Env.SANDBOX:
        raise HTTPException(status_code=500, detail="Sandbox only.")


@router.post("/bearer_token_from_phone", response_model=str)
def get_bearer_token_from_phone_number(  # type: ignore
    phone_number: PhoneNumber,
    _=Depends(is_valid_header),
    __=Depends(is_sandbox),
) -> str:
    token = logic.bearer_token_from_phone_number(phone_number=phone_number)
    return token.decode("utf-8")


@router.post("/bearer_token_from_auth_id", response_model=str)
def get_bearer_token_from_auth_id(  # type: ignore
    auth_id: str,
    _=Depends(is_valid_header),
    __=Depends(is_sandbox),
) -> str:
    token = logic.bearer_token_from_auth_id(auth_id=auth_id)
    return token.decode("utf-8")



import typing as T
import random
import string
import re
from datetime import datetime
from zoneinfo import ZoneInfo
from edge_orm import UNSET, CHANGES
from devtools import pformat
from pydantic import BaseModel


def random_str(n: int) -> str:
    return "".join(random.choices(string.ascii_letters + string.digits, k=n))


def random_str_digits(n: int) -> str:
    return "".join(random.choices(string.digits, k=n))


def operation_name_from_query(q: str) -> str:
    pattern = r"(query|mutation)\s+(\w+)"
    if matches := re.findall(pattern, q):
        return matches[0][1]
    else:
        q = "".join(q.split())
        return q[:20]


NYC_TZ = ZoneInfo("America/New_York")

DEFAULT_TZ = ZoneInfo("America/New_York")


def nyc() -> datetime:
    return datetime.now(tz=NYC_TZ)


def is_value(v: T.Any) -> bool:
    if v is UNSET:
        return False
    if v is None:
        return False
    return True


def camel_to_snake(s: str) -> str:
    return "".join(["_" + c.lower() if c.isupper() else c for c in s]).lstrip("_")


def snake_to_camel(s: str) -> str:
    temp = s.split("_")
    return temp[0] + "".join(ele.title() for ele in temp[1:])


def pretty_str(val: T.Any, tz: T.Optional[ZoneInfo] = None) -> str:
    tz = tz or DEFAULT_TZ
    if isinstance(val, str):
        try:
            val = datetime.fromisoformat(val)
        except ValueError:
            pass
    if isinstance(val, datetime):
        return val.astimezone(tz=tz).strftime("%m/%d/%Y %-I:%M%p %Z")
    if isinstance(val, BaseModel):
        return pformat(val)
    return str(val)


def pretty_price(price: float | int) -> str:
    return f"${price:,.2f}".replace(".00", "")


def changes_to_data(
    changes: CHANGES,
    include_last_updated_at: T.Optional[bool] = True,
    only_include_changed_values: T.Optional[bool] = True,
) -> dict[str, str]:
    LAST_UPDATED_AT_STR = "last_updated_at"
    MAX_STR_LEN = 300

    def to_strikethrough(b: T.Any, a: T.Any) -> str:
        before_str = pretty_str(b)[0:MAX_STR_LEN]
        after_str = pretty_str(a)[0:MAX_STR_LEN]
        return f"~`{before_str}`~\n`{after_str}`"

    data = {}
    for field_name, (before, after) in changes.items():
        if only_include_changed_values:
            if before == after:
                continue
        if field_name == LAST_UPDATED_AT_STR:
            continue
        data[field_name] = to_strikethrough(b=before, a=after)
    if include_last_updated_at:
        if b_and_a := changes.get(LAST_UPDATED_AT_STR):
            (before, after) = b_and_a
            data[LAST_UPDATED_AT_STR] = to_strikethrough(b=before, a=after)
    return data


def make_possessive(name: str) -> str:
    if name.endswith("s"):
        return name + "'"
    else:
        return name + "'s"



from __future__ import annotations
import typing as T
from pydantic import BaseModel
from strawberry.types.types import TypeDefinition
from edge_orm import Node, Resolver
from quilt.vendors.Sentry.context_managers import span

NodeType = T.TypeVar("NodeType", bound=Node)
NodeTypeTemp = T.TypeVar("NodeTypeTemp", bound=Node)


class GQLUtilException(Exception):
    pass


def basemodel_to_straw(straw_cls: T.Type[StrawType], model: BaseModel) -> StrawType:
    """FUTURE check for speed, but for now it's like 1ms for small objs"""
    if model is None:
        raise GQLUtilException("model cannot be None")
    fields_to_fill = {
        f.name for f in straw_cls._type_definition.fields if f.base_resolver is None
    }
    d: dict[str, T.Any] = {}
    for field in fields_to_fill:
        d[field] = getattr(model, field)
    straw = straw_cls(**d)
    straw._node = model
    return straw


class classproperty(property):
    def __get__(self, owner_self: T.Any, owner_cls: T.Any) -> T.Any:  # type: ignore
        return self.fget(owner_cls)  # type: ignore


class StrawMixin(T.Generic[NodeType]):
    _node: NodeType
    _type_definition: TypeDefinition

    class Config:
        node_type: T.Type[NodeType]  # type: ignore
        resolver_type: T.Type[Resolver]

    @property
    def node(self) -> NodeType:
        return self._node

    @classmethod
    def from_node(
        cls: T.Type[StrawType], node: NodeType, use_span: bool = True
    ) -> StrawType:
        with span(op=f"from_node.{cls.__name__}", use=use_span):
            return basemodel_to_straw(straw_cls=cls, model=node)

    @classmethod
    def from_node_or_none(
        cls: T.Type[StrawType], node: NodeType | None, use_span: bool = True
    ) -> StrawType | None:
        if node is None:
            return None
        return cls.from_node(node=node, use_span=use_span)

    @classmethod
    def from_nodes(
        cls: T.Type[StrawType], nodes: list[NodeType], use_span: bool = True
    ) -> list[StrawType]:
        with span(
            op=f"from_nodes.{cls.__name__}",
            description=f"nodes count: {len(nodes)}",
            use=use_span,
        ):
            return [cls.from_node(node, use_span=use_span) for node in nodes]


StrawType = T.TypeVar("StrawType", bound=StrawMixin)  # type: ignore



import typing as T
from pydantic import BaseModel
import strawberry
from enum import Enum


class VerticalAlign(str, Enum):
    TOP = "top"
    CENTER = "center"
    BOTTOM = "bottom"


class Image(BaseModel):
    height: int
    width: int
    url: str
    public_cloudinary_id: str
    vertical_align: VerticalAlign | None = None


strawberry.enum(VerticalAlign)


@strawberry.experimental.pydantic.type(model=Image, name="Image", all_fields=True)
class ImageStraw:
    pass


@strawberry.experimental.pydantic.input(model=Image, name="ImageInput", all_fields=True)
class ImageStrawInput:
    pass


class SmallImage(BaseModel):
    url: str
    public_cloudinary_id: T.Optional[str] = None
    height: T.Optional[int] = None
    width: T.Optional[int] = None


@strawberry.experimental.pydantic.type(
    model=SmallImage, name="SmallImage", all_fields=True
)
class SmallImageStraw:
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['quilt',
 'quilt.context_base',
 'quilt.extensions',
 'quilt.gql_utils',
 'quilt.gql_utils.resolvers',
 'quilt.middlewares',
 'quilt.models',
 'quilt.models.location',
 'quilt.plugins.StrawberryFilters',
 'quilt.scalars',
 'quilt.services.auth',
 'quilt.vendors',
 'quilt.vendors.Cloudinary',
 'quilt.vendors.Firebase',
 'quilt.vendors.GooglePlaces',
 'quilt.vendors.GooglePlaces.tests',
 'quilt.vendors.IPInfo',
 'quilt.vendors.Sentry']

package_data = \
{'': ['*']}

install_requires = \
['devtools',
 'firebase-admin',
 'httpx',
 'phonenumbers',
 'pydantic[email]',
 'sentry-sdk[httpx,pure-eval]>=1.15,<2.0',
 'strawberry-graphql']

setup_kwargs = {
    'name': 'quilt-py',
    'version': '0.5.4',
    'description': '',
    'long_description': 'hello!',
    'author': 'Jeremy Berman',
    'author_email': 'jerber@sas.upenn.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



from graphql.error import GraphQLError
from strawberry.extensions import MaskErrors
import edge_orm
from quilt import DisplayException


def should_mask_error(error: GraphQLError) -> bool:
    # strawberry's validation errors
    if not error.original_error:
        return False
    return not isinstance(
        error.original_error,
        (
            DisplayException,
            PermissionError,
            edge_orm.ExecuteConstraintViolationException,
            edge_orm.PermissionsError,
        ),
    )


mask_errors = MaskErrors(
    error_message="Internal Server Error", should_mask_error=should_mask_error
)

__all__ = ["mask_errors"]



import re
import typing as T
import types
from enum import Enum
from pathlib import Path
from black import format_str, FileMode
import edge_orm
from edge_orm.types_generator.main import stringify_dict

"""NODES"""

NodeType = T.TypeVar("NodeType", bound=T.Type[edge_orm.Node])


cast_to_python_filter_name = {
    "std::uuid": "FilterID",
    "std::str": "FilterString",
    "std::json": "FilterJSON",
    "std::datetime": "FilterDateTime",
    "cal::local_date": "FilterDate",
    "cal::local_time": "FilterTime",
    "std::bool": "FilterBoolean",
    "std::int16": "FilterInt16",
    "std::int32": "FilterInt32",
    "std::int64": "FilterInt64",
    "std::float16": "FilterFloat16",
    "std::float32": "FilterFloat32",
    "std::float64": "FilterFloat64",
}

python_filter_name_to_cast = {
    val: key for key, val in cast_to_python_filter_name.items()
}


def import_str() -> str:
    lst = [
        "from __future__ import annotations",
        "import strawberry",
        "from edge_orm import UnsetType, UNSET",
        "from pydantic import Field",
    ]
    return "\n".join(lst)


def indent_lines(s: str, indent: str = "    ") -> str:
    chunks = s.split("\n")
    return indent + f"\n{indent}".join(chunks)


def build_filter_str_from_node(node_cls: NodeType) -> str:
    fields: dict[str, str] = {}
    print(f"node_cls: {node_cls}")
    for field_name, val in node_cls.EdgeConfig.node_edgedb_conversion_map.items():
        cast = val.base_cast or val.cast
        if filter_str := cast_to_python_filter_name.get(cast):
            fields[field_name] = filter_str
        else:
            # check if enum
            if cast.startswith("default::"):
                enum_name = cast.split("::")[-1]
                fields[field_name] = f"Filter{enum_name}"
            else:
                print("NO MATCH", f"{field_name=}, {cast=}")
                # array<str>... worry about this later
                pass

    # now for links
    for link_name, val in node_cls.EdgeConfig.insert_link_conversion_map.items():
        fields[link_name] = f"NestedFilter{val.cast}"

    field_strs: list[str] = []
    field_strs_straw: list[str] = []
    for field_name, filter_str_val in fields.items():
        field_strs.append(f"{field_name}: {filter_str_val} | UnsetType = UNSET")
        field_strs_straw.append(f"{field_name}: {filter_str_val}Straw | None = UNSET")
    all_fields_str = "\n".join(field_strs)
    all_fields_str_straw = "\n".join(field_strs_straw)

    filter_name = f"Filter{node_cls.__name__}"
    filter_name_straw = f"{filter_name}Straw"
    root_filter = f"""
class {filter_name}(ParseFilterMixin):
    and_: list[{filter_name}] | UnsetType = Field(strawberry.UNSET, alias="and")
    or_: list[{filter_name}] | UnsetType = Field(strawberry.UNSET, alias="or")
    not_: {filter_name} | UnsetType = Field(strawberry.UNSET, alias="not")
{indent_lines(all_fields_str)}


@strawberry.input(name="{filter_name}")
class {filter_name_straw}(AddToResolverMixin):
    and_: list[{filter_name_straw}] | None = strawberry.field(name="and", default_factory=lambda: strawberry.UNSET)
    or_: list[{filter_name_straw}] | None = strawberry.field(name="or", default_factory=lambda: strawberry.UNSET)
    not_: {filter_name_straw} | None = strawberry.field(name="not", default_factory=lambda: strawberry.UNSET)
{indent_lines(all_fields_str_straw)}
    
    class Config:
        pydantic_type = {filter_name}
    """

    nested_filter_name = f"Nested{filter_name}"
    nested_filter_name_straw = f"{nested_filter_name}Straw"

    nested_filter = f"""

class {nested_filter_name}(ParseFilterMixin):
    exists: bool | UnsetType = UNSET
{indent_lines(all_fields_str)}

@strawberry.input(name="{nested_filter_name}")
class {nested_filter_name_straw}:
    exists: bool | None = UNSET
{indent_lines(all_fields_str_straw)}

    class Config:
        pydantic_type = {nested_filter_name}
    """

    order_strs: list[str] = []
    order_strs_straw: list[str] = []
    for field_name, filter_str_val in fields.items():
        ordering_str = "Ordering"
        if filter_str_val.startswith("NestedFilter"):
            ordering_str = "Order" + filter_str_val.split("NestedFilter")[-1]
        order_strs.append(f"{field_name}: {ordering_str} | UnsetType = UNSET")
        order_strs_straw.append(f"{field_name}: {ordering_str}Straw | None = UNSET")

    all_order_str = "\n".join(order_strs)
    all_order_str_straw = "\n".join(order_strs_straw)

    order_name = f"Order{node_cls.__name__}"
    order_name_straw = f"{order_name}Straw"

    order_str = f"""
class {order_name}(ParseFilterMixin):
{indent_lines(all_order_str)}

@strawberry.input(name="{order_name}")
class {order_name_straw}(AddToResolverMixinOrder):
{indent_lines(all_order_str_straw)}

    class Config:
        pydantic_type = {order_name}
    """

    return root_filter + "\n" + nested_filter + "\n" + order_str


def build_node_str(nodes: list[T.Type[edge_orm.Node]]) -> str:
    strs = [build_filter_str_from_node(node) for node in nodes]
    # now do forward refs too
    node_str = "\n".join(strs)
    pattern = "class (\w+)\(ParseFilterMixin\):"
    class_names = re.findall(pattern, node_str)
    print(f"{class_names=}")
    ref_lst: list[str] = [f"{name}.update_forward_refs()" for name in class_names]
    return node_str + "\n" + "\n".join(ref_lst)


def build_node_str_from_module(module: types.ModuleType) -> str:
    unique_nodes = unique_nodes_from_module(module)
    s = build_node_str(unique_nodes)
    return format_str(s, mode=FileMode())


def unique_nodes_from_module(module: types.ModuleType) -> list[T.Type[edge_orm.Node]]:
    node_vals: list[T.Type[edge_orm.Node]] = []
    for d in dir(module):
        val = getattr(module, d)
        try:
            if issubclass(val, edge_orm.Node) and hasattr(val, "EdgeConfig"):
                node_vals.append(val)
        except TypeError:
            pass
    # dedup, take the last one imported
    seen_nodes_by_name: dict[str, T.Type[edge_orm.Node]] = {}
    unique_nodes: set[T.Type[edge_orm.Node]] = set()
    for node in node_vals:
        if seen_node := seen_nodes_by_name.get(node.__name__):
            if issubclass(seen_node, node):
                continue
            else:
                unique_nodes.remove(seen_node)
        seen_nodes_by_name[node.__name__] = node
        unique_nodes.add(node)
    return sorted(list(unique_nodes), key=lambda x: x.__name__)


"""ENUMS"""


def build_filter_str_from_enum(e: T.Type[Enum]) -> str:
    if e.__name__ == "Enum":
        return ""
    enum_name = f"enums.{e.__name__}"
    model_name = f"Filter{e.__name__}"
    model_name_straw = f"{model_name}Straw"
    return f"""
class {model_name}(ParseFilterMixin):
    in_: list[{enum_name}] | UnsetType = Field(alias="in", default_factory=lambda: strawberry.UNSET)
    nin: list[{enum_name}] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: {enum_name} | UnsetType = UNSET
    neq: {enum_name} | UnsetType = UNSET
    gte: {enum_name} | UnsetType = UNSET
    gt: {enum_name} | UnsetType = UNSET
    lte: {enum_name} | UnsetType = UNSET
    lt: {enum_name} | UnsetType = UNSET

@strawberry.input(name="{model_name}")
class {model_name_straw}:
    in_: list[{enum_name}] | None = strawberry.field(name="in", default_factory=lambda: strawberry.UNSET)
    nin: list[{enum_name}] | None = UNSET
    exists: bool | None = UNSET
    eq: {enum_name} | None = UNSET
    neq: {enum_name} | None = UNSET
    gte: {enum_name} | None = UNSET
    gt: {enum_name} | None = UNSET
    lte: {enum_name} | None = UNSET
    lt: {enum_name} | None = UNSET
"""


def build_enum_str(enums: list[T.Type[Enum]]) -> str:
    print(f"{enums=}")
    strs = [build_filter_str_from_enum(e) for e in enums]
    return "\n".join(strs)


def build_enum_str_from_module(module: types.ModuleType) -> str:
    enum_vals: list[T.Type[Enum]] = []
    for d in dir(module):
        val = getattr(module, d)
        try:
            if issubclass(val, Enum):
                enum_vals.append(val)
        except TypeError:
            pass
    s = build_enum_str(sorted(list(set(enum_vals)), key=lambda x: x.__name__))
    return format_str(s, mode=FileMode())


class GeneratorException(Exception):
    pass


def build_default_filters_path_import_str(default_filters_import_path: str) -> str:
    if not default_filters_import_path.endswith(".default_filters"):
        raise GeneratorException(
            "default_filters_import_path must end with `.default_filters`"
        )
    imports = [
        "ParseFilterMixin",
        "AddToResolverMixin",
        "AddToResolverMixinOrder",
        "FromDictMixin",
        "FilterID",
        "FilterIDStraw",
        "FilterDateTime",
        "FilterDateTimeStraw",
        "FilterDate",
        "FilterDateStraw",
        "FilterTime",
        "FilterTimeStraw",
        "FilterJSON",
        "FilterJSONStraw",
        "FilterString",
        "FilterStringStraw",
        "FilterBoolean",
        "FilterBooleanStraw",
        "FilterInt64",
        "FilterInt64Straw",
        "FilterInt16",
        "FilterInt16Straw",
        "FilterInt32",
        "FilterInt32Straw",
        "FilterFloat16",
        "FilterFloat16Straw",
        "FilterFloat32",
        "FilterFloat32Straw",
        "FilterFloat64",
        "FilterFloat64Straw",
        "Ordering",
        "OrderingStraw",
        "DirectionEnum",
        "NullsOrderingEnum",
    ]
    return f"from {default_filters_import_path} import {', '.join(imports)}"


def generate_filters(
    enums_module: types.ModuleType,
    db_module: types.ModuleType,
    default_filters_import_path: str,
    output_path: Path,
    enums_path: str,
) -> None:
    enums_s = build_enum_str_from_module(enums_module)
    nodes_s = build_node_str_from_module(db_module)

    # update forward refs for these nodes -> so get all classes and update the forward refs
    class_names = re.findall("class (\w+)\(", nodes_s)
    non_straw_class_names = [c for c in class_names if not c.endswith("Straw")]
    update_forward_refs_str = "\n".join(
        [f"{c}.update_forward_refs()" for c in non_straw_class_names]
    )

    mapping = {f"{c}": c for c in non_straw_class_names}
    mapping_str = "FilterOrderByName: dict[str, ParseFilterMixin] = " + stringify_dict(
        mapping, stringify_value=False
    )

    strs = [
        import_str(),
        build_default_filters_path_import_str(default_filters_import_path),
        enums_path,
        enums_s,
        nodes_s,
        update_forward_refs_str,
        mapping_str,
    ]
    s = format_str("\n".join(strs), mode=FileMode())
    with open(output_path, "w") as output_file:
        output_file.write(s)


def main() -> None:
    from app.dbs import enums, db

    generate_filters(
        enums,
        db,
        "from app.plugins.StrawberryFilters.default_filters import *",
        Path("app/app/dbs/edgedb/gen/filters.py"),
        enums_path="from app.dbs import enums",
    )


if __name__ == "__main__":
    main()
